//+------------------------------------------------------------------+
//|                                            Qore-Astro-Events.mq4 |
//|                                                        QoreLogic |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright "QoreLogic"
#property link      "http://www.phuu.com"

#property indicator_chart_window
#define DELIM ";"

#include <qorelib.mqh>

extern int timezoneOffset = 2;
extern int periodMask = PERIOD_MN1; //;
extern int maxEvents = 1000;

int dateTime;
string aspect1, aspect2, aspect3, mCustomEffect;
double id;

string lines[];


//----
int elcnt = 6;
string parentTag = "astro-events";
string childTag = "event";
string tags[] = { "id", "aspect1", "aspect2", "aspect3", "datetime", "timestamp" };
//----

int		DebugLevel = 2;

double 	ExtMapBuffer0[];	// Contains (minutes until) each news event
double 	ExtMapBuffer1[];	// Contains only most recent and next news event ([0] & [1])
double 	ExtMapBuffer2[];	// Contains impact value for most recent and next news event

string	sUrl = "http://qore.phuu.com/astroevents.php";
int 	xmlHandle;
int 	BoEvent, finalend, end, i;
int		begin;
string 	mainData[100][6]; // Array size of 2nd dimension must match elcnt
int 	minsTillNews;
string 	sData, csvoutput;
int		tmpMins;
int		idxOfNext;
int		dispMinutes[2];
string 	dispTitle[2], 
		dispCountry[2], 
		dispImpact[2], 
		dispForecast[2], 
		dispPrevious[2];
string	dispSeparator1 = "===================";

static bool FirstRun = true;
static int	PrevMinute = -1;


int init()
{
   if (Period() <= periodMask) {
      readXML();
   }  
   
   return(0);
}

void drawChart(string id, string aspect1, string aspect2, string aspect3, string timestamp, string& lines[], int i) {

      int dateTime = StrToInteger(timestamp);
      dateTime = dateTime + 3600*timezoneOffset;
      lines[i] = aspect1 + " | " + aspect2 + " | " + aspect3 + " | " + dateTime;
      //Print(lines[i]);
      
      ObjectDelete(lines[i]);
      if(!ObjectCreate(lines[i], OBJ_VLINE, 0, dateTime, 0)) {
         Print("error: can't create label_object! code #",GetLastError());
         return(0);
      }
      //ObjectSet(lines[i], OBJPROP_STYLE, STYLE_DOT);
      ObjectSet(lines[i], OBJPROP_COLOR, 0x444444);
      //ObjectSet(lines[i], OBJPROP_LEVELWIDTH, 5);
         
      if (aspect2 == "Conj")
         ObjectSet(lines[i], OBJPROP_COLOR, 0x008888);
      if (aspect2 == "Oppos")
         ObjectSet(lines[i], OBJPROP_COLOR, 0x885555);
      if (aspect2 == "Qucnx")
         ObjectSet(lines[i], OBJPROP_COLOR, 0x444444);
      if (aspect2 == "Sesqu")
         ObjectSet(lines[i], OBJPROP_COLOR, 0x444444);
      if (aspect2 == "Sqr")
         ObjectSet(lines[i], OBJPROP_COLOR, 0x444488);
      if (aspect2 == "SSqr")
         ObjectSet(lines[i], OBJPROP_COLOR, 0x444444);
      if (aspect2 == "SSxt")
         ObjectSet(lines[i], OBJPROP_COLOR, Aqua);
      if (aspect2 == "Sxtil")
         ObjectSet(lines[i], OBJPROP_COLOR, 0x444444);
      if (aspect2 == "Trine")
         ObjectSet(lines[i], OBJPROP_COLOR, 0x448844);
}

int deinit()
{
   for (int i = 0; i < ArraySize(lines); i++) {   
      ObjectDelete(lines[i]);
   }
   return(0);
}

int start()
{
   return(0);
}

///-/- ----------------------------------------------------------------- -/-///
void readXML() {
	int 		newsIdx = 0;
	int 		nextNewsIdx = -1;
	int 		next;
	string 		myEvent;
	bool 		skip;
	datetime 	newsTime;
	
   string event[][];

	// If we are not logging, then do not output debug statements either
	if (!EnableLogging)
		DebugLevel = 0;
				
	string xmlFileName = Month() + "-" + Day() + "-" + Year() + "-" + "QoreAstro.xml";
	Print("STARTED");
	
	if (FirstRun || Minute() == 0)
	//if (true)
	{
		FirstRun = false;

		if (DebugLevel > 1)
			Print("sUrl == ", sUrl);
		
		if (DebugLevel > 0)
			Print("Grabbing Web, url = ", sUrl);
	
		sData = getResponse(sUrl);

		if (DebugLevel > 0)
		{
			Print("Opening XML file...\n");
			Print(sData);
		}

		xmlHandle = FileOpen(xmlFileName, FILE_BIN|FILE_READ|FILE_WRITE);
		if (xmlHandle < 1)
		{
			if (DebugLevel > 0)
				Print("Can\'t open new xml file, the last error is ", GetLastError());
			return(false);
		}
		FileWriteString(xmlHandle, sData, StringLen(sData));
		FileClose(xmlHandle);

		if (DebugLevel > 0)
			Print("Wrote XML file...\n");

		// THIS BLOCK OF CODE DONATED BY WALLY TO FIX THE RESOURCE ERROR
		//--- Look for the end XML tag to ensure that a complete page was downloaded ---//
		end = StringFind(sData, "</"+parentTag+">", 0);

		if (end <= 0)
		{
			Alert("Error - Web page download was not complete!");
			return(false);
		}
		//-------------------------------------------------------------------------------//
	}

	// = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
	// Perform remaining checks once per minute
	if (Minute() == PrevMinute)
		return (true);
	PrevMinute = Minute();
	Print("NEW MINUTE...");

	// Open the log file (will not open if logging is turned off)
	OpenLog("QoreAstro");

	// Init the buffer array to zero just in case
	ArrayInitialize(ExtMapBuffer0, 0);
	ArrayInitialize(ExtMapBuffer1, 0);
	
	// Open the XML file
	xmlHandle = FileOpen(xmlFileName, FILE_BIN|FILE_READ);
	if (xmlHandle < 0)
	{
		Print("Can\'t open xml file: ", xmlFileName, ".  The last error is ", GetLastError());
		return(false);
	}
	if (DebugLevel > 0)
		Print("XML file open must be okay");
	
	// Read in the whole XML file
	// Ave file length == ~7K, so 65536 should always read whole file
	sData = FileReadString(xmlHandle, 65536);	
	FileClose(xmlHandle);

	// Get the currency pair, and split it into the two countries
	string pair = Symbol();
	string cntry1 = StringSubstr(pair, 0, 3);
	string cntry2 = StringSubstr(pair, 3, 3);
	if (DebugLevel > 0)
		Print("cntry1 = ", cntry1, "    cntry2 = ", cntry2);
	
	if (DebugLevel > 0)
		Log("Weekly calendar for " + pair + "\n\n");

	// -------------------------------------------------
	// Parse the XML file looking for an event to report
	// -------------------------------------------------
	tmpMins = 10080;	// (a week)
	BoEvent = 0;
    
   string sTags[], eTags[];
   ArrayResize(sTags, elcnt);
   ArrayResize(eTags, elcnt);
   for (i=0; i < ArraySize(tags); i++) {
      sTags[i] = "<"+tags[i]+">";
      eTags[i] = "</"+tags[i]+">";
   }
   
   int j = 0;
	string dateTime[];
	string aspect3[];
	string aspect2[];
	string aspect1[];
	string id[];
	while (j <= maxEvents)
	{
		BoEvent = StringFind(sData, "<"+childTag+">", BoEvent);
		if (BoEvent == -1)
			break;
			
		BoEvent += elcnt;
		next = StringFind(sData, "</"+childTag+">", BoEvent);
		myEvent = StringSubstr(sData, BoEvent, next - BoEvent);
		BoEvent = next;
		
		begin = 0;
		skip = false;
		for (i=0; i < elcnt; i++)
		{
			mainData[newsIdx][i] = "";
			next = StringFind(myEvent, sTags[i], begin);
			
			// Within this event, if tag not found, then it must be missing; skip it
			if (next == -1)
				continue;
			else
			{
				// We must have found the sTag okay...
				begin = next + StringLen(sTags[i]);			// Advance past the start tag
				end = StringFind(myEvent, eTags[i], begin);	// Find start of end tag
				if (end > begin && end != -1)
				{
					// Get data between start and end tag
					mainData[newsIdx][i] = StringSubstr(myEvent, begin, end - begin);
				}
			}
		}		
		
		ArrayResize(dateTime, j+1);
		ArrayResize(aspect3, j+1);
		ArrayResize(aspect2, j+1);
		ArrayResize(aspect1, j+1);
		ArrayResize(id, j+1);
		ArrayResize(lines, j+1);
		dateTime[j] = mainData[newsIdx][5];
		aspect3[j] = mainData[newsIdx][3];
		aspect2[j] = mainData[newsIdx][2];
		aspect1[j] = mainData[newsIdx][1];
		id[j] = mainData[newsIdx][0];

		drawChart(id[j], aspect1[j], aspect2[j], aspect3[j], dateTime[j], lines,  j);

	   j++;
   }
}

